#!/usr/bin/python

######################################################
#              Nunchuktest 0.1 by Cadex              #
#                                                    #
#    Tiny tool that shows a few bytes that seem to   #
#          contain the state of the Nunchuk           #
#                                                    #
#             Requires Python an PyBluez             #
######################################################

import bluetooth
import thread
import time

# Sockets and status
receivesocket = bluetooth.BluetoothSocket(bluetooth.L2CAP)
controlsocket = bluetooth.BluetoothSocket(bluetooth.L2CAP)
status = "Disconnected"

# Connect to the Wiimote at bluetooth address <address>
def connect(address):
	global status
	receivesocket.connect((address, 0x13))
	controlsocket.connect((address, 0x11))
	if receivesocket and controlsocket:
		print "[Connected to Wiimote at address " + address + "]"
		status = "Connected"
		thread.start_new_thread(receivethread, ())
	else:
		print "[Could not connect to Wiimote at address " + address + "]"

# Disconnect from the Wiimote
def disconnect():
	global status
	status = "Disconnecting"
	while status == "Disconnecting":
		wait(1)
	print "[Disconnected]"

# Try to discover a Wiimote
def discover():
	print "[Searching for Wiimotes...]"
	address = None
	bluetoothdevices = bluetooth.discover_devices(lookup_names = True)
	for bluetoothdevice in bluetoothdevices:
		if bluetoothdevice[1] == "Nintendo RVL-CNT-01":
        		address = bluetoothdevice[0]
			print "[Found Wiimote at address " + address + "]"
	if address == None:
		print "[No Wiimotes discovered. Press buttons 1 + 2 at the same time prior to running the script in order to switch Wiimote to discoverable mode]"
	return address
def tobin(i):
  b = ''
  while i > 0:
    j = i & 1
    b = str(j) + b
    i >>= 1
  if len(b) < 8:
    pad = 8 - len(b)
    b = '0'*pad + b
  return b

# Thread that listens for incoming data
def receivethread():
	global status
	receivesocket.settimeout(0.01)
	while status == "Connected":
		try:
			data = receivesocket.recv(23)
			if len(data):
				if ord(data[1]) == 0x37:
					ncd = data[0x2:]
					nunchukdata = []
					for b in ncd:
					  pb = int(ord(b))
					  nb = (pb ^ 0x17)+0x17
					  nunchukdata.append(nb)

					print "Nunchuk data len (%d):   " % len(data),
					for byte in nunchukdata:
				#		print str(byte).encode("hex").upper(),
						print tobin(byte),
					print
				else:
				  print "fds" + str(len(data))
		except bluetooth.BluetoothError:
			pass
	receivesocket.close()
	controlsocket.close()
	status = "Disconnected"

# Send <data2> to the Wiimote
def send(data2):
	data = ""
	for datum in data2:
		data += chr(datum)
	controlsocket.send(data)

# Wait <millis> milliseconds
def wait(millis):
	time.sleep(millis / 1000.0)

# Main program code
# Try to discover a Wiimote instead of using a certain bluetooth address
address = discover()
if address != None:
	# Try to connect to Wiimote
	connect(address)
	if status == "Connected":
	        wait(10)
		print "s1"
		send([0x52, 0x16, 0x04, 0xa4, 0x00, 0x40, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])

		wait(100)
		print "s2"
		send([ 0x52, 0x12, 0x04, 0x37 ])
		wait(100)
		send([0x52, 0x16, 0x04, 0xa4, 0x00, 0x40, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])
		wait(100000)
		while(False):
			send([0x52, 0x17, 0x04, 0xa4, 0x00, 0x00, 0x00, 0x10])
			wait(5000)
		# Disconnect from the Wiimote
		disconnect()

