## WMD Command Mapper
## Last visit: 2006-12-27

# The Command Mapper receives user-generated events, such as:
##  WM_BT        WiiMote Button Events (*)
##  WM_EXT_BT    External Controller Button Events ( )
##  GES_ACC      Accelerometer gesture recognized (-)
##  GES_ANALOG   Analog axis gesture recognized (-)
#
# It then looks up the command associated to that event, and executes it.
# 
# It should also recognize the following mode changes:
#
##  Lock Keys    Reserve certain keys and disable their commands ( )
##  Release Lock Release locked keys ( )
##  Load Map     Load a new command map ( )
##  Previous Map Reload the previous command map (-)
##  Display Map  Access map information for display ( )
#
#   (*) Implemented   ( ) To Implement    (-) Low Priority

from wmd.Common import *
from wmd.EventBridges.uinputKeymap import *

import time

# This maps button presses (and possibly other event types, if we subscribe to them as well) to
# commands defined in the config file
class CommandMapper:
  # Right now we have to define valid commands manually. Those valid commands will
  # map directly to a function name, in the form: do_COMMAND
  # WMDPower and WMDMode need to be dealt with.
  # The only use of WMDPower at this time is for something that should be done with a Lock Keys
  # And WMDMode is being used like WMDPower should
  valid_commands = [ 'quit', 'key', 'click', 'WMDPower', 'WMDMode' ]
  
  def __init__( self, cf, ev ):
    self.cf = cf
    self.ev = ev
    self.cMap = self.cf['commandMap']

    self.ev.subscribe( WM_BT, self.ev_wm_bt )
    self.ev.subscribe( WM_EXT_BT, self.ev_wm_ext_bt )

  def ev_wm_ext_bt( self, ev ):
    (bt, state) = ev
    print "External Controller Button Press: %s, %s" % (bt, state)
    self.ev_wm_bt( ev )
 
  def ev_wm_bt( self, ev ):
    bt = ev[0]
    state = ev[1]
    
    if bt in self.cMap:
      cmName = self.cMap[bt][0]
      cmParms = self.cMap[bt][1]

      if cmName in self.valid_commands:
        funcname = "do_" + cmName 
        func = getattr( self, funcname )
	func( state, cmParms )
      else:
        log( LOG_ERR, "Invalid command for button" )
    else:
      log( LOG_ERR, "Button doesn't exist" )
      
  def do_quit( self, state, parms ):
    self.ev.send( UI_INFO, "BYE BYE WMD WILL EXIT IN A SECOND" )
    time.sleep(1)
    self.ev.send( EV_SHUTDOWN, '' )

  def do_key( self, state, parms ):
    keys = parms
    self.command_key( state, keys )

  def do_click( self, state, parms ):
    # Really need to re-enable the click lock here
    # A global timer system would be a good idea

    self.ev.click_lock() 
    key = parms
    self.command_key( state, key )

  # And to make this do what it was supposed to, iterate between modes
  def do_WMDPower( self, state, parms ):
    if parms == 'increase':
      self.ev.send( WMDPOWER, 0.2 )
    if parms == 'decrease':
      self.ev.send( WMDPOWER, -0.2 )
  
  # This will go away?
  def do_WMDMode( self, state, parms ):
    if parms == 'next' and state == 'DOWN':
      self.ev.send( UI_SWITCH, 'next' )

  def command_key( self, evtype, keyname ):
    keycodes = []

    if type(keyname) == type([]):
      for kn in keyname:
        keycodes.append( self.map_keyname( kn ) )
    else:
      keycodes.append( self.map_keyname( keyname ) )

    for kc in keycodes:
      if evtype == "DOWN":
        self.ev.send( EVBR_KEYDOWN, kc )
      elif evtype == "UP":
	self.ev.send( EVBR_KEYUP, kc )

  def map_keyname( self, keyname ):
    keycode = codeMaps["EV_KEY"].toNumber( keyname )
    return keycode
