# WMD 0.2
from wmd.Common import *

# Motion sensing manager

# It's quite a mess. It is right now hardcoded to process extension controller data differently.
# this is quite critical, to enable routing of such data

# It should be possible to connect any source of force to this system, and also, to log force data
# First, write integrated force analyzers, and then move them out, until a higher level of genericity
# is achieved through natural evolution.


class MSManager:
  force_log_maxlen = 20
  force_disturbance = 200
  force_log = []
  axes = ['x','y','z']
  quash = { 'duration': 5, 'age': 0 }
  
  Wei = 0.90
  f_avg = []

  def __init__( self, cf, ev ):
    self.cf = cf
    self.ev = ev

    # Here, should we be doing some form of averaging?
    # We need to have event types, like RAW_ACC, but also event sources types, like WM, WM_EXT, LOGFILE_STREAM
    self.ev.subscribe( WM_ACC, self.ev_wm_acc )
    self.ev.subscribe( WM_EXT_ACC, self.ev_wm_ext_acc )

    # First, we need to buffer the force data into a log
    # This is needed so that backwards-looking analysis can be performed

    # We need to gather some information on the force data

    # Peaks, Zeros, Disturbance Averages, Averages, Orientation, Tilt, Frequency..
    # Each simple analyzer is a derivative of an analysis class?

    # Analyzers/two means of operation:
    #    Continuous analysis (updates itself at each event)
    #    Look-back analysis (uses a log)

    # This is two-dimensional data, because of the important time factor.
    # On three axes

    # Will need axis rotation and separation eventually, but not now.
    
    # Need to move the calibration stuff out of here and into a common library, it appears.

    # Same thing with the pointer smoothing and normalization code
    # This should be shared better

  def ev_wm_ext_acc( self, force ):
    f = []

    for ax in self.axes:
      f.append( force[ax] )

    if not self.f_avg:
      self.f_avg = f
      return

    for i in range(3):
      a = self.f_avg[i]
      n = f[i]
      na = ( self.Wei * a ) + ( (1.0-self.Wei) * n )
      self.f_avg[i] = na

    self.get_tilt()

  def ev_acc( self, force, source ):
    pass

  def acc_averager( self, force, source, distance ):
    return force_average

  def acc_logger( self, force, source ):
    return force_log

  # An interative algorithm is better, because it can be called back if needed, fed by a log


  def ev_wm_acc( self, force ):
    # Route to wiimote gravity mouse
    self.ev_wm_ext_acc( force )
    self.sense_force( force )

  def sense_force( self, force ):
    if len( self.force_log ) > self.force_log_maxlen:
      self.force_log.pop(0)

    self.force_log.append(force)

    disturbances = {}
    last_axv = {}
    i = 0
    
    axes = ['x','y','z']

    for f in self.force_log:
      for ax in axes:
	v = f[ax]
	if i == 0:
	  last_axv[ax] = v
	  disturbances[ax] = 0
	else:
	  disturbances[ax] += abs(last_axv[ax]-v)
	  last_axv[ax] = v
      i = i + 1

    disturbed_axes = []

    for ax in axes:
      v = disturbances[ax]
      if v > self.force_disturbance:
	disturbed_axes.append(ax)
    
    if self.quash['age'] > 0:
      self.quash['age'] -= 1
    elif disturbed_axes:
      log(LOG_FORCE, "I SENSE A GREAT DISTURBANCE IN THE FORCE..." + str(disturbances))
      self.quash['age'] = self.quash['duration']
      log(DEBUG_FORCE, "Force log: " + str(self.force_log))
    

  def get_tilt( self ):
    x = self.f_avg[0]
    y = self.f_avg[1]

    print "tilt (x,y) = (%d, %d)" % (x,y)
    
    axisMax = 160   # Maximum value
    axisMin = 100  # Minimum value
    
    #axisMax = 160   # Maximum value - nunchuk
    #axisMin = 80    # Minimum value - nunchuk

    axisDz = 2    # Dead zone around the edges
    axisNz = 2     # Neutral zone at the center

    axisCenter = ( axisMax + axisMin ) / 2

    defaultJoystickAxisCalibration = {
      'min': axisMin + axisDz,               # Minimum absolute value in edge dead zone
      'max': axisMax - axisDz,               # Maximum absolute value in edge dead zone
      'cMin': axisCenter - axisNz,           # Minimum absolute value in center neutral zone
      'cMax': axisCenter + axisNz,           # Maximum absolute value in center neutral zone
      'rMin': axisCenter - axisNz - axisDz,  # Number of steps between dead and neutral zones
      'rMax': axisCenter - axisNz - axisDz   # Number of steps (range) between dead and neutral zones
    } 

    nx = self.normalizeJoystickAxis( x, defaultJoystickAxisCalibration ) 
    ny = self.normalizeJoystickAxis( y, defaultJoystickAxisCalibration ) 

    #print "x=%.3f, y=%.3f, nx=%.3f, ny=%.3f" % ( x, y, nx, ny )

    Axes = { 'x': nx, 'y': ny }
    self.ev.send( REL_TILT, Axes )


  def normalizeJoystickAxis( self, axVal, axCal ):
    """Normalize an analog axis to a [ -1, 1 ] range using calibration values provided"""
    # Calibration looks like axCal = { 'min': 10, 'max': 53, 'cMin': 28 , 'cMax': 36, 'rMin': 18, 'rMax': 18}
    
    # Convert to float
    axVal += 0.0
    
    normAxVal = 0.0

    ndiv = 0.1
    nadd = 100
    
    if axVal <= axCal['min']:
      normAxVal = -1.0
    elif axVal >= axCal['max']:
      normAxVal = 1.0
    elif axVal > axCal['cMin'] and axVal < axCal['cMax']:
      normAxVal = 0.0
    elif axVal <= axCal['cMin']:
      normAxVal = 0.0 - ( ( axCal['cMin'] - axVal) / (axCal['rMin']+nadd) )
      normAxVal /= ndiv
    elif axVal >= axCal['cMax']:
      normAxVal = 0.0 + ( ( axVal - axCal['cMax'] ) / (axCal['rMax']+nadd) )
      normAxVal /= ndiv
    else:
      pass
      log( LOG_ERR, "UNMET NORMALIZATION!!" )
      
    return normAxVal


