from wmd.Common import *

class PyGCommon:
  caption = "WMD 0.1.3"

  winwidth = 1024
  winheight = 768 + 100

  comheight = 100

  width = winwidth
  height = winheight - comheight

  bgcolor = ( 255, 255, 255 )
  fontheight = 20
  
  mmcolor = (150,150,150)
  fontpadding = 5
  prev_ui_info_rect = None

  def __init__( self, ev, evHandlers ):
    self.ev = ev
    self.evHandlers = evHandlers

    if self.try_import():
      # Initialize pygame screen with selected height, width, background colour; set background colour immediately
      self.pygame.init()
      self.init_font()

      self.screen = self.pygame.display.set_mode( (self.winwidth, self.winheight), self.pygame.RESIZABLE )
      self.pygame.display.set_caption( self.caption )
      self.screen.fill( self.bgcolor )
      self.pygame.display.flip()

      # Register for event processing of these event types only. However, other events still seem to get through.
      pgevlist = [ self.pygame.VIDEORESIZE, self.pygame.VIDEOEXPOSE ]
      self.pygame.event.set_allowed( pgevlist )

      evHandlers.append( [ WM_CLOCK_BEAT, self.clock_beat ] )
      evHandlers.append( [ UI_INFO, self.ev_ui_info ] )
      for ev in evHandlers:
        evId = ev[0]
	callback = ev[1]
        self.ev.subscribe( evId, callback )

      return 1

  def ev_ui_info( self, msg ):
    self.writeline( msg )

  def writeline( self, msg, pos=0 ):
    if self.font:
      textStr = msg
      mmcolor = self.mmcolor
      text = self.font.render(textStr, 1, mmcolor)
      textpos = text.get_rect( centerx = self.width/2,
                               top = self.height + (self.fontheight + self.fontpadding) * pos )

      if self.prev_ui_info_rect:
        self.screen.fill(self.bgcolor, self.prev_ui_info_rect)

      self.screen.blit(text, textpos)

      if self.prev_ui_info_rect:
        self.pygame.display.update( self.prev_ui_info_rect )
      else:
        self.pygame.display.update( textpos )

      self.prev_ui_info_rect = textpos

  def stop( self ):
    for ev in self.evHandlers:
      evId = ev[0]
      callback = ev[1]
      self.ev.unsubscribe( evId, callback )

    self.pygame.display.quit()
    self.screen = None
    self.pygame = None

  def init_font( self ):
    if self.pygame.font:
      try:
	self.font = self.pygame.font.Font("wmd/VeraMono.ttf", self.fontheight)
      except:
	try:
	  self.font = self.pygame.font.Font(None, self.fontheight)
	except:
	  self.font = 0
	  log(LOG_ERR, "Cannot load font")

  def try_import( self ):
    try:
      import pygame
      self.pygame = pygame
      return 1
      
    except:
      log(LOG_ERR, "Could not import pygame. Set DISABLE_PYGAME=1")
      return 0

  clockdiv = 100
  divbeat = 0

  def clock_beat( self, beatno ):
    self.proc_events()

    if self.divbeat == self.clockdiv:
      self.divbeat = 0
      self.do_div_beat()
    else:
      self.divbeat += 1

  def do_div_beat( self ):
    self.writeline( "Commands:  2=NextDisplay   Home=Quit", 1 )
   
  def proc_events( self ):
  ## This processes events, and allows us to resize the window
    pgev = self.pygame.event.poll()
    while pgev:
      if pgev:
	if pgev.type == self.pygame.QUIT:
	  self.ev.send( EV_SHUTDOWN, '' ) 
      pgev = self.pygame.event.poll()
