# First, we import required constants and the basic logging system common to every WMD Class
from wmd.Common import *

# Then, we import WMD PyGame support, which will initialize the display.
from wmd.UI.PyGCommon import PyGCommon

class PyGExtAnalog(PyGCommon):
  # Default values are 1024x768, the real size of the IR sensor.

  # This will be used as the title of your window.
  caption = "WMD Attachment Controller Analog Axis Grapher - Joysticks and Triggers"

  dotcolor = ( 40, 40, 120 ) # What colour should the dots by painted?

  AxisDotColors = {
    'rt': ( 0, 0, 255 ),     # Right Trigger
    'lt': ( 15, 100, 200 ),  # Left Trigger
    'rj': ( 255, 0, 0 ),     # Right Joystick
    'lj': ( 200, 100, 15 )   # Left Joystick
  }

  sf = 6                       # Size scaling factor, dot_radius = sf * dot_intensity

  prevdots = []
  update_list = []

  def __init__( self, ev ):
    evHandlers = [ [ WM_EXT_ANALOG, self.on_Ext_Analog_Report ] ]
    PyGCommon.__init__( self, ev, evHandlers )

  def on_Ext_Analog_Report( self, Axes ):
    # Here we erase the dots we drew during the last WM_IR event
    if self.prevdots:
      for dot in self.prevdots:
	self.drawdot( dot, color=self.bgcolor )
      self.prevdots = []

    if Axes:
      dots = self.getAnalogDots( Axes )
      for axis, dot in dots.items():
        dotcolor = self.dotcolor

	if self.AxisDotColors.has_key( axis ):
	  dotcolor = self.AxisDotColors[ axis ]

        self.drawdot( dot, color=dotcolor )
	self.prevdots.append( dot )

    self.refresh()
    self.proc_events()

  def getAnalogDots( self, Axes ):
    dots = {}
    ds = 2      # Dot Size
    srx = 1024

    if 'rt' in Axes:
      dots['rt'] = [ srx/4*3, srx*Axes['rt'], ds ]

    if 'lt' in Axes:
      dots['lt'] = [ srx/4*1, srx*Axes['lt'], ds ]

    if 'rx' in Axes and 'ry' in Axes:
      (sx, sy) = self.scaleNormDot( Axes['rx'], Axes['ry'] )
      dots['rj'] = [ sx, sy, ds ]

    if 'lx' in Axes and 'ly' in Axes:
      (sx, sy) = self.scaleNormDot( Axes['lx'], Axes['ly'] )
      dots['lj'] = [ sx, sy, ds ]

    return dots

  def scaleNormDot( self, nx, ny, rMin=-1.0, rMax=1.0, srx=1024, sry=768 ):
    if rMin == -1.0:
      nx += rMax
      nx /= 2

      ny += rMax
      ny /= 2

    sy = int( ny * sry )
    sx = int( nx * srx )

    return [ sx, sy ]

  # Here we draw a dot. To erase one, draw it in the background color
  def drawdot( self, dot, color=dotcolor ):
    (x, y, s) = dot # We're not going to be using v
    rs = s * self.sf

    self.pygame.draw.circle( self.screen, color, (x, y), rs)

    # We want to redraw only a small rectangle that comprises the dot
    updRect = self.pygame.Rect( x-rs, y-rs, rs*2, rs*2 )
    # And we want to hold these redraws until we've drawn every dot for the cycle
    self.update_list.append( updRect )

  # At the end of the cycle, we update all modified regions of the screen
  def refresh( self ):
    if self.update_list:
      self.pygame.display.update( self.update_list )
      self.update_list = []

