# First, we import required constants and the basic logging system common to every WMD Class
from wmd.Common import *

# Then, we import WMD PyGame support, which will initialize the display.
from wmd.UI.PyGCommon import PyGCommon

class PyGIRDisplay(PyGCommon):
  # Default values are 1024x768, the real size of the IR sensor.

  # This will be used as the title of your window.
  caption = "WMD 0.1.3 PyGIRDisplay - ** Point your WiiMote at an IR Source! ** (unplug any extension controller)"

  bgcolor = ( 0, 0, 0 )        # Background color: Black.
  dotcolor = ( 255, 230, 230 ) # What colour should the dots by painted? Reddish white.
  sf = 6                       # Size scaling factor, dot_radius = sf * dot_intensity

  prevdots = []
  update_list = []

  def __init__( self, ev ):
    evHandlers = [ [ WM_IR, self.on_IR_Report ] ]
    PyGCommon.__init__( self, ev, evHandlers )

  def on_IR_Report( self, IR_Report ):
    ( dots, dots_on ) = IR_Report

    # Here we erase the dots we drew during the last WM_IR event
    if self.prevdots:
      for dot in self.prevdots:
	self.drawdot( dot, color=self.bgcolor )
      self.prevdots = []

    if dots_on and len( dots ):
      # Now we draw the newly received dots
      for dot in dots:
        if len(dot) == 4:
	  dot_is_valid = dot[2]  # We're not going to use x, y and s for now
	  # But only the valid ones
	  if dot_is_valid:
	    self.drawdot( dot )
	    self.prevdots.append( dot )

    self.refresh()
    self.proc_events()

  # Here we draw a dot. To erase one, draw it in the background color
  def drawdot( self, dot, color=dotcolor ):
    (x, y, v, s) = dot # We're not going to be using v
    rs = s * self.sf

    self.pygame.draw.circle( self.screen, color, (x, y), rs)

    # We want to redraw only a small rectangle that comprises the dot
    updRect = self.pygame.Rect( x-rs, y-rs, rs*2, rs*2 )
    # And we want to hold these redraws until we've drawn every dot for the cycle
    self.update_list.append( updRect )

  # At the end of the cycle, we update all modified regions of the screen
  def refresh( self ):
    if self.update_list:
      self.pygame.display.update( self.update_list )
      self.update_list = []

