# WMD 0.2
from wmd.Common import *

# The code here is fairly good, but the calibration stuff should be elsewhere
# I need a calibration system

class ExtensionController:
  def __init__( self ):
    self.PrevButtonStates = {}

  def decode_bytes( self, Bytes ): return [ ( ( Byte ^ 0x17 ) + 0x17 ) & 0xFF for Byte in Bytes ] 
  def invert_bytes( self, Bytes ): return [ Byte ^ 0xFF for Byte in Bytes ]

  def detect_button_state_changes( self, ButtonStates ):
    # On the very first run, we need to do prepare the PrevButtonStates dict
    if not self.PrevButtonStates:
      for bt in self.ButtonNames:
        self.PrevButtonStates[ bt ] = 0
      return 1

    # Then, we should iterate over each button
    # And test if the button state has changed
    for Button, bt_state in ButtonStates.iteritems():
      # Button is in [ 'R', 'D', 'Lt', '-', 'H', '+', 'Rt', 'Zl', 'B', 'Y', 'A', 'X', 'Zr', 'L', 'U' ]
      # NewState is in [ 'UP', 'DOWN' ]

      #bt_state = ButtonStates[ Button ]

      # We want to detect state transition, so if the state hasn't changed
      # We're skipping over to the next button
      if self.PrevButtonStates[ Button ] == bt_state:
        continue
      
      if bt_state == 1:
        NewState = 'DOWN'
      else:
        NewState = 'UP'

      Button = self.BTPrefix + Button
        
      BtEvent = [ Button, NewState ]
    
      self.ev.send( WM_EXT_BT, BtEvent )
    
    # When we're finished.
    self.PrevButtonStates = ButtonStates

  def NormalizeAxes( self, AnalogAxes ):
    """Take absolute AnalogAxes positions and normalize them to [ -1, 1 ]"""

    # Ranges: lx, ly are 0-63
    #         rx, ry, rt, lt are 0-31
    
    # Sample ranges for the Classic Controller axes
    # Indicates the need for a "Dead Zone", or, better yet, calibration
    # I understand that the Classic Controller does have calibration data that can be read
    # But the format is not understood completely, I think
    # ycros =   { 'lx': [7, 57], 'ly': [5, 56] 'rx': [2, 27], 'ry': [3, 28], 'lt': [4, 31], 'rt': [4, 30] }
    # hernick = { 'lx': [5, 31, 56], 'ly': [6, 32, 58], 'rx': [3, 15, 29], 'ry': [2, 15, 27], 'lt': [3, 31], 'rt': [5, 31] }

    # The dead zone seems to be different from one axis to the other
    # So.. is 8-56 a good value? 9-55 or 10-54 might be more conservative
    # Let's use 10-54 for users who haven't calibrated
    # I wonder if we can individually identify unique Classic Controllers
    
    # Neutral around 30-32
    
    # Let's try default calibration values at 10-54, center DZ at 28-34
    # Let's convert the axis data to a float range of [ -1, 1 ] for easy maths
    # We have to find the transition point though.. Calibration would be very useful for this
    # But not essential at this time
    
    axisMax = 63   # Maximum value
    axisMin = 0    # Minimum value
    axisDz = 12    # Dead zone around the edges
    axisNz = 4     # Neutral zone at the center

    if self.ECName == "Nunchuk":
      axisMax = 255 
      axisDz = 20

    axisCenter = ( axisMax + 1 ) / 2
    
    # Okay, we obviously have two kind of axes...
    # Those that have both positive and negative values, that range [ -1, 1 ], that is lx, ly, rx, ry
    # And those that only have positive values, that range [ 0, 1 ], that is lt and rt

    defaultTriggerAxisCalibration = {
      'min': axisMin + axisDz,               # Minimum absolute value in edge dead zone
      'max': axisMax - axisDz,               # Maximum absolute value in edge dead zone
      'r': axisMax - 2 * axisDz              # Number of steps (range) between dead and neutral zones
    }

    defaultJoystickAxisCalibration = {
      'min': axisMin + axisDz,               # Minimum absolute value in edge dead zone
      'max': axisMax - axisDz,               # Maximum absolute value in edge dead zone
      'cMin': axisCenter - axisNz,           # Minimum absolute value in center neutral zone
      'cMax': axisCenter + axisNz,           # Maximum absolute value in center neutral zone
      'rMin': axisCenter - axisNz - axisDz,  # Number of steps between dead and neutral zones
      'rMax': axisCenter - axisNz - axisDz   # Number of steps (range) between dead and neutral zones
    } 

    NormalizedAxes = {}
    
    for ax in AnalogAxes:
      axVal = AnalogAxes[ax]          # Absolute value of axis

      if ax in self.JoystickAxes:
        axCal = defaultJoystickAxisCalibration    # Axis calibration data
        normAxVal = self.normalizeJoystickAxis( ax, axVal, axCal )
      elif ax in self.TriggerAxes:
        axCal = defaultTriggerAxisCalibration
	normAxVal = self.normalizeTriggerAxis( ax, axVal, axCal )
      else:
        pass
	#print "AXIS NOT FOUND"

      NormalizedAxes[ ax ] = normAxVal
      
    return NormalizedAxes
   
  def normalizeTriggerAxis( self, ax, axVal, axCal ):
    """Normalize an analog axis to a [ 0, 1 ] range using calibration values provided"""
    # Calibration looks like axCal = { 'min': 10, 'max': 53, 'r': 40 }
    
    axVal += 0.0
    normAxVal = 0.0
    
    if axVal <= axCal['min']:
      normAxVal = 0.0
    elif axVal >= axCal['max']:
      normAxVal = 1.0
    elif axVal > axCal['min'] and axVal < axCal['max']:
      r = axCal['r']
      v = axVal - axCal['min']
      normAxVal = v/r
    else:
     log( LOG_ERR, "UNMET TRIGGER NORM!!" )

    return normAxVal

  def normalizeJoystickAxis( self, ax, axVal, axCal ):
    """Normalize an analog axis to a [ -1, 1 ] range using calibration values provided"""
    # Calibration looks like axCal = { 'min': 10, 'max': 53, 'cMin': 28 , 'cMax': 36, 'rMin': 18, 'rMax': 18}
    
    # Convert to float
    axVal += 0.0
    
    normAxVal = 0.0
    
    if axVal <= axCal['min']:
      normAxVal = -1.0
    elif axVal >= axCal['max']:
      normAxVal = 1.0
    elif axVal > axCal['cMin'] and axVal < axCal['cMax']:
      normAxVal = 0.0
    elif axVal <= axCal['cMin']:
      normAxVal = 0.0 - ( ( axCal['cMin'] - axVal ) / axCal['rMin'] )
    elif axVal >= axCal['cMax']:
      normAxVal = 0.0 + ( ( axVal - axCal['cMax'] ) / axCal['rMax'] )
    else:
      pass
      log( LOG_ERR, "UNMET NORMALIZATION!!" )
      
    return normAxVal


class ClassicControllerState(ExtensionController):
  ECName = "Classic"
  BTPrefix = "C"

  ButtonNames = [ 'R', 'D', 'Lt', '-', 'H', '+', 'Rt', 'Zl', 'B', 'Y', 'A', 'X', 'Zr', 'L', 'U' ]

  JoystickAxes = [ 'lx', 'ly', 'rx', 'ry' ]
  TriggerAxes = [ 'rt', 'lt' ]
 
  def __init__( self, ev ):
    self.ev = ev
    ExtensionController.__init__( self )
    #self.ev.subscribe( WM_EXT_BT, self.cc_ev_test )
    #self.ev.subscribe( WM_EXT_ANALOG, self.cc_ev_test )
    
  # This function receives raw reports, 6 bytes long, at about 100FPS when in mode 0x37
  # the cc needs to be plugged in from the start
  def process_slice( self, slb ):
    decB = self.decode_bytes( slb )

    CCButtonStates = self.decode_buttons( decB )
    CCAnalogAxes = self.decode_analog_axes( decB )
    
    # Analog axes must be normalized first
    CCNormalizedAxes = self.NormalizeAxes( CCAnalogAxes )

    # And then neutral axes must be filtered out, and the rest are sent as events
    self.sendActiveAxisEvents( CCNormalizedAxes )

    # Detect button state transitions and send events when transitions occur
    self.detect_button_state_changes( CCButtonStates )
   
  def cc_ev_test( self, ev ):
    pp( ev )

  def sendActiveAxisEvents( self, axes ):
    """Only send events containing axes that aren't presently neutral"""
    activeAxes = {}

    for ax, val in axes.items():
      if val != 0.0:
        activeAxes[ ax ] = val

    if activeAxes:
      if 'lx' in activeAxes and not 'ly' in activeAxes:
        activeAxes['ly'] = 0.0
      elif 'ly' in activeAxes and not 'lx' in activeAxes:
        activeAxes['lx'] = 0.0

      if 'rx' in activeAxes and not 'ry' in activeAxes:
        activeAxes['ry'] = 0.0
      elif 'ry' in activeAxes and not 'rx' in activeAxes:
        activeAxes['rx'] = 0.0

      self.ev.send( WM_EXT_ANALOG, activeAxes )

  def decode_buttons( self, B ):
    # The buttons themselves are encrypted differently, so we decode them into a two-byte list
    wmButtonBytes = self.invert_bytes( B[4:6] ) 
    
    # And this button map corresponds to each of the decrypted bytes
    #             0    1     2    3    4    5    6     7
    buttonByteMasks = [ [ 'R', 'D', 'Lt', '-', 'H', '+', 'Rt', '_' ],   # First byte
                        [ 'Zl', 'B', 'Y', 'A', 'X', 'Zr', 'L', 'U' ] ]  # Second byte
    # _ isn't actually a button though
    # We need to filter it out

    # Now we do something. That is undocumented.
    # It's actually quite useless.
    # This is what we're here to fix.

    CCButtonStates = {}  # We'll build a nice map of the current button states.
    byteLen = 8

    for i in range( len( buttonByteMasks ) ):
      byteMask = buttonByteMasks[i]
      buttonByte = bf( wmButtonBytes[i] )
      for bit in range( byteLen ):
        buttonName = byteMask[ bit ]
        buttonState = buttonByte[ byteLen - 1 - bit ]
        if buttonName != '_':
          CCButtonStates[buttonName] = buttonState
	
    return CCButtonStates

  def decode_analog_axes( self, decB ):
    bfB = [ bf(B) for B in decB ]

    # We extract the bit slices FTW, which is allowed by the fact that those are BitFields
    lx = bfB[0][0:6]
    ly = bfB[1][0:6]
    ry = bfB[2][0:5]
    rt = bfB[3][0:5]

    # Here we do things differently, using DesktopMa's algorithm.
    # This is like magic. Don't try to understand what it does.
    rx = (decB[0] & 0xC0) >> 3 | (decB[1] & 0xC0) >> 5 | (decB[2] & 0x80) >> 7
    lt = (decB[2] & 0x60) >> 2 | (decB[3] & 0xE0) >> 5

    log(DEBUG_EXTCON, "lx=%d, ly=%d, rx=%d, ry=%d, lt=%d, rt=%d" % (lx, ly, rx, ry, lt, rt) )

    # So, here we've decoded the Classic Controller state
    # We have:
    
    # CCButtonStates, a dictionary that contains the current state of all buttons
    # like CCButtonStates = { 'R': 0, 'A': 0, 'B': 1, ... }
    
    # And lx, ly, rx, ry, rt, lt
    # Left Joystick X, Y, Right Joystick X, Y, Right Trigger, Left Trigger
    # Those are relative events
    
    CCAnalogAxes = {
      'lx': lx,
      'ly': ly,
      'rx': rx,
      'ry': ry,
      'rt': rt,
      'lt': lt,
    }
    
    # Appears that if we don't do that, we get a mix of longs and ints
    # This code I just added should make everything ints
     

    for ax in CCAnalogAxes:
      CCAnalogAxes[ax] = int( CCAnalogAxes[ax] )

    #print CCAnalogAxes
    # These four axes have less precision.. Let's scale everything up to the same 6-bit scale, 0-63 range
    FiveBitAxes = [ 'rx', 'ry', 'lt', 'rt' ]
    for ax in FiveBitAxes:
      CCAnalogAxes[ax] = 2 * CCAnalogAxes[ax]
      
    return CCAnalogAxes

  def calibrate( self, D ):
    D = self.decode_bytes(D)

    m = 'min'; M = 'max'; C = 'center'

    lx = {}; ly = {}; rx = {}; ry = {}; lt = {}; rt = {}

    ( lx[m], lx[C], lx[M] ) = D[0:3]
    ( ly[m], ly[C], ly[M] ) = D[3:6]
    ( rx[m], rx[C], rx[M] ) = D[6:9]
    ( ry[m], ry[C], ry[M] ) = D[9:12]
    ( lt[m], lt[M] ) = D[12:14]
    ( rt[m], rt[M] ) = D[14:16]

    JS = {
      'lx': lx,
      'ly': ly,
      'rx': rx,
      'ry': ry,
      'lt': lt,
      'rt': rt
    }

    for J in JS:
      if J[0] == 'r':
        d = 8 
      elif J[0] == 'l':
        d = 4
      else:
        print J[0]
      v = JS[J]
      if v.has_key( C ):
        vax = [ v[m], v[C], v[M] ]
      else:
        vax = [ v[m], v[M] ]
	d = 8
      vax = [ int(x/d) for x in vax ]
      print " // " + J[0] + " " + str(d) + " ",
      print J + ": ",
      print vax,
      

class NunchukState(ExtensionController):
  ECName = "Nunchuk"
  BTPrefix = "N"

  ButtonNames = [ 'Z', 'C' ] 
  JoystickAxes = [ 'rx', 'ry' ]

  def __init__( self, ev ):
    ExtensionController.__init__( self )
    self.ev = ev

  def process_slice( self, slb ):
    pass

  def process_slice( self, slb ):
    decB = self.decode_bytes( slb )

    ButtonStates = self.decode_buttons( slb[5] )
    (AnalogAxes, Force) = self.decode_analog_axes( decB )
    
    # Analog axes must be normalized first
    NormalizedAxes = self.NormalizeAxes( AnalogAxes )

    # And then neutral axes must be filtered out, and the rest are sent as events
    self.sendActiveAxisEvents( NormalizedAxes )

    self.ev.send( WM_EXT_ACC, Force )

    # Detect button state transitions and send events when transitions occur
    self.detect_button_state_changes( ButtonStates )

  def decode_buttons( self, B ):
    Z = not( B & 1 )
    C = ( not ( ( B >> 1) & 1 ) ) ^ Z
    
    ButtonStates = {
      'Z': Z,
      'C': C
    }

    return ButtonStates
    
  def decode_analog_axes( self, decB ):
    AnalogAxes = {
      'rx': decB[0],
      'ry': decB[1],
    }

    Force = {
      'x': decB[2],
      'y': decB[3],
      'z': decB[4]
    }
    
    return [ AnalogAxes, Force ]

  def sendActiveAxisEvents( self, axes ):
    """Only send events containing axes that aren't presently neutral"""
    activeAxes = {}

    for ax, val in axes.items():
      if val != 0.0:
        activeAxes[ ax ] = val

    if activeAxes:
      if 'rx' in activeAxes and not 'ry' in activeAxes:
        activeAxes['ry'] = 0.0
      elif 'ry' in activeAxes and not 'rx' in activeAxes:
        activeAxes['rx'] = 0.0

      self.ev.send( WM_EXT_ANALOG, activeAxes )

  def calibrate( self, D ):
    D = self.decode_bytes( D )
    print "Nunchuk calibration bytes: ",
    print D
    pass
