import string, time
from copy import copy
from wmd.Common import *
from wmd.Wiimote.WiimoteReports import WiimoteReports, ReportSlices
from wmd.Wiimote.ExtensionControllers import ClassicControllerState, NunchukState
import struct

import pprint
pp = pprint.pprint

class ReportParser:
  """Receives report packets from the Wiimote
     Classifies and then slices them
     Individually processes and dispatches the slices
     """

  def __init__( self, ev, wmstate ):
    self.ev = ev
    self.wmstate = wmstate

  def parse( self, report ):
    rep_header = ord(report[0:1])
    rep_type = ord(report[1:2])
    rep_payload = report[2:]

    if rep_header != 0xa1:
      log( LOG_ERR, "Invalid report header" )

    rep_def = WiimoteReports[ rep_type ]
    rep_def_len = rep_def[0]

    if len(rep_payload) != rep_def_len:
      log( LOG_ERR, "Report length doesn't match expected definition value" )

    #print "Recv rep type %x (%s)" % ( rep_type, rep_def[1] )
    self.split_report( rep_payload, rep_def )

  def split_report( self, rep_payload, rep_def ):
    rep_def_slicenames = rep_def[2]
    for slicename in rep_def_slicenames:
      slicedef = ReportSlices[slicename]
      slice_len = slicedef[0]
      slice_spos = slicedef[3]
      slice_epos = slicedef[4]
      if slice_epos-slice_spos != slice_len:
        log( LOG_ERR, "Slice length computation error" )
      
      slice_data = rep_payload[slice_spos:slice_epos]
      
      slice_bl = []
      for sb in slice_data:
        b = int(ord(sb))
	slice_bl.append(b)

      funcname = "slice_" + slicename
      func = False
      try:
        func = getattr( self, funcname )
      except:
        log( LOG_ERR, "Cannot find handler function %s for slicename %s" % ( funcname, slicename ) )

      if func:
	func( slice_bl )
        try:
	  pass
	except:
	  log( LOG_ERR, "Cannot call func %s" % funcname )
    
  def slice_BUTTONS( self, slb ):
    btdat = 256*slb[0] + slb[1]
    self.wmstate.which_buttons( btdat )

  def slice_ACCELEROMETER( self, slb ):
    force = {
      'x': slb[0],
      'y': slb[1],
      'z': slb[2]
    }

    self.ev.send( WM_ACC, force )

  def slice_IR_0x33( self, slb ):
    """Parses the common 12-byte IR slice
       The heavy lifting is done by preprocess_dots"""

    dots = [
      slb[0:3],
      slb[3:6],
      slb[6:9],
      slb[9:12],
    ]

    (pdots, dots_on) = self.preprocess_dots(dots)
    self.ev.send( WM_IR, [pdots, dots_on] )

  def slice_IR_0x37( self, slb ): #untested # poorly tested.. mode switch seems to jam
    """Unpacks IR dots from 10-byte IR reports
       The only way to know if dots are valid is by testing if all bits are 1
       And there is no way to know the size of the dots in mode 0x37"""

    ir = slb
    s = 4

    irdots = [

     [ ir[0] | ((ir[2] & 0x30) << 4),
       ir[1] | ((ir[2] & 0xC0) << 2) ],
 
     [ ir[3] | ((ir[2] & 0x03) << 8),
       ir[4] | ((ir[2] & 0x0C) << 6) ],
 
     [ ir[5] | ((ir[7] & 0x30) << 4),
       ir[6] | ((ir[7] & 0xC0) << 2) ],
 
     [ ir[8] | ((ir[7] & 0x03) << 8),
       ir[9] | ((ir[7] & 0x0C) << 6) ]

    ]

    dots_on = 0
    for i in range( len( irdots ) ):
      v = 1
      if irdots[i][0] == 1023 and irdots[i][1] == 1023:
        v = 0
      else:
	dots_on += 1

      irdots[i].append( v )
      irdots[i].append( s )

    self.ev.send( WM_IR, [irdots, dots_on] )
    #print irdots


  def slice_STATUS( self, slb ):
    stat = bf( slb[0] )
    s = {}
    s['attachment'] = stat[1]
    s['continuous'] = stat[3]
    s['leds'] = stat[4:8]
    self.process_stats( s )

  def slice_BAT( self, slice ):
    bat = slice
    self.process_bat( bat )
  
  def slice_EXT_0x37( self, slb ):
    if self.wmstate.ext_device_name in [ 'Classic', 'Nunchuk' ]:
      self.wmstate.ext_device.process_slice( slb )

  def slice_MEM_WRITE_STATUS( self, slb ):
    pass

  def slice_BATTERY( self, slb ):
    pass

  def slice_MEM_READ_DATA( self, slb ):
    self.wmstate.incoming_ram_data( slb )

  def slice_STATUS_UNKNOWN( self, slb ):
    pass

  def preprocess_dots( self, dots ):
    """Here, we take an hex string containing the raw bytes with information
       on the dots. We then decode this information into usable variables.
       We're going to do ugly bitfield manipulation here. It seems to be
       reliable, but I wouldn't attempt to modify it.
       
       Here are the variables we're going to extract for each dot:
         lx, ly: least significant bytes of x and y
	 mx, my: most significant two bits of x and y
         x:      horizontal position
	 y:      vertical position
	 s:      dot intensity measurement
         v:      dot validity
       """
    dots_sum = [0,0]
    dots_on = 0
    pdots = [ [0,0,0,0], [0,0,0,0] ]

    for dn in range(2):
      for dax in range(3):
	dotc_val = dots[dn][dax]
	dots[dn][dax] = dotc_val
	dots_sum[dn] += dotc_val

    for dn in range(2):
      lx = dots[dn][0]
      ly = dots[dn][1]
      ib = bf( dots[dn][2] )
      s = ib[0:3]
      mx = bf( ib[4:6] )
      my = bf( ib[6:8] )
      x = bf(lx)
      y = bf(ly)
      x[8] = mx[0]
      x[9] = mx[1]
      y[8] = my[0]
      y[9] = my[1]
      pdots[dn][0] = int(x)
      pdots[dn][1] = int(y)
      if dots_sum[dn] != 255*3:
        v = 1
	dots_on += 1
	log(DEBUG_IR2, "dn = %u, lx = %u, ly = %u, mx = %u, my = %u, s = %u, v = %u, x = %u, y = %u" % (dn, lx, ly, mx, my, s, v, x, y))
        pdots[dn][2] = 1
      pdots[dn][3] = s

    if dots_on:
      return [pdots, dots_on]
    else:
      return [0,0]

  def process_stats( self, s ):
    if s['attachment']:
      log( LOG_INFO,  "EXTCON_ATTACH EVENT v=%s" % str( s['attachment'] ) )
      self.wmstate.attach_ext_device( )
    elif self.wmstate.ext_device_name and not s['attachment']:
      log( LOG_INFO,  "EXTCON_DETACH EVENT v=%s" % str( s['attachment'] ) )
      self.wmstate.detach_ext_device()
      
      #self.ev_ATTACH_CHECK(1)
    pass

  def process_bat( self, b ):
    pass

# This contains stuff that should be shared with ExtensionControllers...
# There is duplication in the button detection code, which is nicer in ExtensionControllers.py
class WiimoteState:
  """This contains the present state of the Wiimote; buttons, battery, etc."""

  valid_ext_device_names = [ 'Classic', 'Nunchuk' ]
  ext_device_name = ''
  ext_device = None
  on_read_ram_callbacks = []

  def __init__( self, ev, wmmode ):
    self.ev = ev
    self.wmmode = wmmode

  # These are the button mappings
  buttonmap = {
    '2': 0x0001,
    '1': 0x0002,
    'B': 0x0004,
    'A': 0x0008,
    '-': 0x0010,
    'H': 0x0080,
    'L': 0x0100,
    'R': 0x0200,
    'D': 0x0400,
    'U': 0x0800,
    '+': 0x1000
  }

  buttonstates = {}
  prevbuttonstates = {}
  for bt in buttonmap:
    buttonstates[bt] = 0
    prevbuttonstates[bt] = 0

  # Here, we extract the button state indicated by the last WiiMote Report
  # This gets called at ~100Hz
  # We use a completely different principle to decode Classic Controller Data
  # But this function uses buttonmap found a few lines up
  def which_buttons( self, rawbtd ):
    btps = []

    if rawbtd != 0:
      for bt in self.buttonmap:
	btk = self.buttonmap[bt]
	if btk & rawbtd:
	  btps.append(bt)

    self.update_button_states( btps )

  # Receives a list of buttons current pressed from which_buttons
  def update_button_states( self, btps ):
    bts_up = []
    bts_down = []

    for bt in self.buttonstates:
      self.prevbuttonstates[bt] = self.buttonstates[bt]
      self.buttonstates[bt] = 0

    for bt in btps:
      self.buttonstates[bt] = 1
   
    for bt in self.prevbuttonstates:
      if self.prevbuttonstates[bt] and not self.buttonstates[bt]:
	self.bt_notify(bt, "UP")
	bts_up.append(bt)
      if not self.prevbuttonstates[bt] and self.buttonstates[bt]:
	self.bt_notify(bt, "DOWN")
	bts_down.append(bt)
    
    if bts_down:
      log(LOG_BTN, "Buttons down: " + str(bts_down))
    if bts_up:
      log(LOG_BTN, "Buttons up: " + str(bts_up))

  def bt_notify( self, bt, state ):
    self.ev.send( WM_BT, [bt, state] )

  def attach_ext_device( self ):
    if self.ext_device:
      #print "ext_device already attached"
      return
      
    # We'll have to call this function upon receiving a 0x20 report 
    # That indicates attachment of an external device
      
    # We have to reply to the WiiMote in order to activate the external device
    self.wmmode.accept_attachment()

    # We'll soon receive a reply that'll tell us what kind of device we're dealing with
    self.read_extcont_type()
      
  def detach_ext_device( self ):
    self.ext_device_name = None
    self.ext_device = None
    self.wmmode.disconnect_attachment()

  def incoming_ram_data( self, data ):
    callback = self.on_read_ram_callbacks.pop(0)
    callback( data )

  def on_read_ram( self, callback, addr_high, addr_low, read_len, a0=0, a1=0 ):
    self.on_read_ram_callbacks.append( callback )
    self.wmmode.read_ram( addr_high, addr_low, read_len, a0, a1 )

  def read_calibration( self ):
    self.on_read_ram( self.read_calibration_callback, 0, 0x16, 8 )

  def read_calibration_callback( self, data ):
  # Calibration Data is being ignored

    print "Wiimote Calibration Bytes: ",
    print data

  def read_ext_calibration( self ):
    self.on_read_ram( self.read_ext_calibration_callback, 0, 0x20, 16, a0=0x04, a1=0xa4 )

  def read_ext_calibration_callback( self, data ):
    D = data[3:20]

    if len(D) != 16:
      print "wrong data len %d for ext_calibration data" % len(D)
      return
    
    if self.ext_device_name in [ "Classic", "Nunchuk" ]:
      self.ext_device.calibrate( D )
    else:
      return

  def read_extcont_type( self ):
    self.on_read_ram( self.read_extcont_type_callback, 0, 0xfe, 2, a0=0x04, a1=0xa4)

  def read_extcont_type_callback( self, data ):
    off = data[2]
    ct1 = data[3]
    ct2 = data[4]

    if off == 0xfe and ct1 == ct2:
      if ct1 == 0xfd:
        ext_device_name = 'Classic'
      elif ct1 == 0xfe:
        ext_device_name = 'Nunchuk'
      else:
        log( LOG_ERR, "Unknown device type %x" % ct1 )
    else:
      log( LOG_ERR, "Invalid memory read during controller attachment" )

    if ext_device_name in self.valid_ext_device_names:
      self.ext_device_name = ext_device_name

      # Then, let's instantiate an object representing the state of the external device
      if ext_device_name == 'Classic':
        msg = "Attaching Classic Controller"
	self.ext_device = ClassicControllerState( self.ev )
      elif ext_device_name == 'Nunchuk':
        msg = "Attaching Nunchuk"
	self.ext_device = NunchukState( self.ev ) 

      self.read_ext_calibration()

      log( LOG_INFO, msg)
      self.ev.send( UI_INFO, msg )
