#!/usr/bin/python

# 'SLICE_ID': [ SLICE_BYTE_LENGTH, SLICE_CHAR, SLICE_DESCRIPTION, SLICE_START_POSITION, SLICE_END_POSITION, [ VALID_IN_REPORTS ] ]

ReportSlices = {
  'BUTTONS': [ 2, 'B', 'Core Buttons', 0, 2, [ 0x20, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37 ] ],
  'STATUS': [ 1, 'S', 'Wiimote Status', 2, 3, 0x20 ],
  'STATUS_UNKNOWN': [ 2 , '0', 'Two Unknown Status Report Bytes', 3, 5, 0x20 ],
  'BATTERY': [ 1, 'L', 'Battery Level', 5, 6, 0x20 ],
  'MEM_READ_DATA': [ 19, 'R', 'Data Read from Memory ', 2, 21, 0x21 ],
  'MEM_WRITE_STATUS': [ 4, 'W', 'Status of Write to Memory', 0, 4, 0x22 ],
  'ACCELEROMETER': [ 3, 'A', 'Accelerometer Measurements', 2, 5, [ 0x31, 0x33, 0x35, 0x37 ] ],
  'EXT_0x32': [ 8, 'E', 'Extension Controller', 2, 10, 0x32 ],
  'EXT_0x34': [ 19, 'E', 'Extension Controller', 2, 21, 0x34 ],
  'EXT_0x35': [ 16, 'E', 'Extension Controller', 5, 21, 0x35 ],
  'EXT_0x36': [ 9, 'E', 'Extension Controller', 12, 21, 0x36 ],
  'EXT_0x37': [ 6, 'E', 'Extension Controller', 15, 21, 0x37 ],
  'EXT_0x3d': [ 21, 'E', 'Extension Controller', 0, 21, 0x3d ],
  'IR_0x36': [ 10, 'I', 'IR Camera Dots', 2, 12, 0x36 ],
  'IR_0x37': [ 10, 'I', 'IR Camera Dots', 5, 15, 0x37 ],
  'IR_0x33': [ 12, 'I', 'IR Camera Dots', 5, 17, 0x33 ],
  'INTERLEAVED_BUTTONS_ACCELEROMETER': [ 3, 'i', 'Interleaved Core Buttons and Accelerometer Measurements', 0, 3, [ 0x3e, 0x3f ] ],
  'INTERLEAVED_IR': [ 18, 'I', 'Interleaved IR Camera Dots', 3, 21, [ 0x3e, 0x3f ] ]
}


# REPORT_ID: [
#   REPORT_BYTE_LENGTH, REPORT_DESCRIPTION,
#   [ 'SLICE_NAME', 'SLICE_NAME', ... ],
#   REPORT_SAMPLE
# ],

WiimoteReports = {
  0x20: [
    6, "Wiimote Status",
    [ 'BUTTONS', 'STATUS', 'BATTERY', 'STATUS_UNKNOWN' ],
    '(a1) 20 BB BB SS 00 00 LL'
  ],

  0x21: [
    21,  "Read Memory Data",
    [ 'MEM_READ_DATA' ],
    '(a1) 21 BB BB RR RR RR RR RR RR RR RR RR RR RR RR RR RR RR RR RR RR RR' #!!!!!! No tested example
  ],

  0x22: [
    4,   "Write Memory Status",
    [ 'MEM_WRITE_STATUS' ],
    '(a1) 22 WW WW WW WW' #!!!!!!! No tested example
  ],

  0x30: [
    2,   "Core Buttons",
    [ 'BUTTONS' ],
    '(a1) 30 BB BB'
  ],

  0x31: [
    5,   "Core Buttons and Accelerometer",
    [ 'BUTTONS', 'ACCELEROMETER' ],
    '(a1) 31 BB BB AA AA AA'
  ],

  0x32: [
    10,  "Core Buttons with 8 Extension bytes",    #!!!!!WIKI ERR says 16
    [ 'BUTTONS', 'EXT_0x32' ],
    '(a1) 32 BB BB EE EE EE EE EE EE EE EE'
  ],

  0x33: [
    17,  "Core Buttons and Accelerometer with 12 IR bytes",
    [ 'BUTTONS', 'ACCELEROMETER', 'IR_0x33' ],
    '(a1) 33 BB BB AA AA AA II II II II II II II II II II II II'
  ],

  0x34: [
    21,  "Core Buttons with 19 Extensions bytes",
    [ 'BUTTONS', 'EXT_0x34' ],
    '(a1) 34 BB BB EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE'
  ],

  0x35: [
    21,  "Core Buttons and Accelerometer with 16 Extension bytes",
    [ 'BUTTONS', 'ACCELEROMETER', 'EXT_0x35' ],
    '(a1) 35 BB BB AA AA AA EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE'
  ],

  0x36: [
    21,  "Core Buttons with 10 IR bytes and 9 Extension bytes",
    [ 'BUTTONS', 'IR_0x36', 'EXT_0x36' ],
    '(a1) 36 BB BB II II II II II II II II II II EE EE EE EE EE EE EE EE EE'
  ],

  0x37: [
    21,  "Core Buttons and Accelerometer with 10 IR bytes and 6 Extension bytes",
    [ 'BUTTONS', 'ACCELEROMETER', 'IR_0x37', 'EXT_0x37' ],
    '(a1) 37 BB BB AA AA AA II II II II II II II II II II EE EE EE EE EE EE' 
  ],

#  0x38: [
#    21,  "?? Buttons | Motion Sensing Report | IR",
#    [ ],
#    ''
#  ],

  0x3d: [
    21,  "21 Extension bytes",
    [ 'EXT_0x3d' ],
    '(a1) 3d EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE EE' #!!!! example wrong on wiki page? (a1) 37..
  ],

  0x3e: [
    21,  "Interleaved Core Buttons and Accelerometer with 36 IR bytes 1/2",
    [ 'INTERLEAVED_BUTTONS_ACCELEROMETER', 'INTERLEAVED_IR' ],
    '(a1) 3e ii ii ii II II II II II II II II II II II II II II II II II II'
  ],

  0x3f: [
    21,  "Interleaved Core Buttons and Accelerometer with 36 IR bytes 2/2",
    [ 'INTERLEAVED_BUTTONS_ACCELEROMETER', 'INTERLEAVED_IR' ],
    '(a1) 3f ii ii ii II II II II II II II II II II II II II II II II II II'
  ]
}

#0x11:    1 	  Player LEDs,  Force Feedback
#0x12: 	2 	Report type / ID
#0x13: 	1 	IR Sensor Enable
#0x14: 	1 	Enable speaker
#0x15: 	1 	Controller status
#0x16: 	21 	Write data
#0x17: 	6 	Read data
#0x18: 	21 	Speaker data
#0x19: 	1 	Mute speaker
#0x1a: 	1 	IR Sensor Enable 2




# We're only going to be running this when running WiimoteReports.py directly
class ProtocolConsistencyTester:
  """This validates and cross-checks information contained in the WiimoteReports and ReportSlices data structures"""
  def __init__( self, reps, slices ):
    self.reps = reps
    self.slices = slices

  def run_tests( self ):
    self.t_reps_len()
    self.t_slices_len()
    self.t_slice_to_rep_map()
    self.t_rep_to_slice_map()
    self.t_rep_examples()

  # This one is ugly, but it tests the example and slice lengths for consistency
  def t_rep_examples( self ):
    for ri in self.reps:
      r = self.reps[ri]
      rl = r[0] 
      re = r[3]

      rel = len(re)
      t_rel = (2*(rl+2)) + ((rl+2)-1) + 2
      
      if rel == 0:
        print "ri 0x%x has no example" % ri
        continue	

      if rel != t_rel:
        print "for ri 0x%x, rl %s, rel %d != t_rel %d" % (ri, rl, rel, t_rel)

      if re[0:5] != "(a1) ":
        print "example doesn't start right..."
	print re[0:5]

      if int(ord( re[5:7].decode("hex") )) != ri:
        print "invalid ri %x" % ( ri )
      
      # Here we're going to test if the example bytes match the length we define in the slice definitions
      ire = re[8:]
      lire = ire.split( " " )

      sbtc = {}
      sbtc_spos = {}
      pos = 0
      for sbt in lire:
        sbtb = sbt[0]
	if sbtc.has_key(sbtb):
  	  sbtc[sbtb] += 1
	else:
	  sbtc[sbtb] = 1
	  sbtc_spos[sbtb] = pos
	pos += 1

      t_sbtc = {}
      sirs = r[2]
      for siri in sirs:
        s = self.slices[siri]
	sl = s[0]
	sbtb = s[1]
	spos = s[3]
	epos = s[4]
	t_sbtc[sbtb] = sl
	# We check if the slice definition start and end position are consistent with slice length
	if spos != sbtc_spos[sbtb]:
	  print "spos != sbtc_spos for ri 0x%x" % (ri)
	# We check if start and end position in the example do match the slice definition
	if (epos - spos) != sl:
	  print "epos(%d) - spos(%d) != sl(%d) for siri %s" % (epos, spos, sl, siri)

      if sbtc != t_sbtc:
        print "t_sbtc != sbtc for ri 0x%x" % (ri)
        print t_sbtc
        print sbtc


  def t_slice_to_rep_map( self ):
    for si in self.slices:
      s = self.slices[si]
      strm = s[5]
      if type(strm) == type(1):
        strm = [strm]
      for stri in strm:
        rts = self.reps[stri][2]
	try:
	  rtsi = rts.index( si )
	except:
	  print "str unmatch for si %s, stri 0x%x, rts %s" % ( si, stri, rts )

  def t_rep_to_slice_map( self ):
    for ri in self.reps:
      r = self.reps[ri]
      rtsm = r[2]
      for rtsi in rtsm:
        rts = self.slices[rtsi][5]
	if type(rts) == type(1):
	  rts = [rts]
  	try:
	  rts.index( ri )
	  #print "str match for ri %s, rtsi %s" % ( ri, rtsi )
	except:
	  print "str unmatch for ri %x, rtsi %s, rts %s" % ( ri, rtsi, rts )

  def t_reps_len( self ):
    REP_LEN = 4
    for ri in self.reps:
      r = self.reps[ri]
      rl = len(r)
      if rl != REP_LEN:
        print "WARN: Report id %s: %s is of len %d" % ( str(ri), str(r), rl )

  def t_slices_len( self ):
    SLICE_LEN = 6
    for si in self.slices:
      s = self.slices[si]
      sl = len(s)
      if sl != SLICE_LEN:
        print "WARN: Slice id %s: %s is of len %d" % ( str(si), str(s), sl )

if __name__ == "__main__":
  pct = ProtocolConsistencyTester( WiimoteReports, ReportSlices )
  print "Running ProtocolConsistencyTester test suite now."
  print "Any message between the lines indicates an error in the protocol definition."
  print "-" * 80
  pct.run_tests( )
  print "-" * 80
  print "End of tests."
